import { writable } from "svelte/store";
import { Router, stringify } from "./vendor";
const cache = {};
const baseTag = document.getElementsByTagName("base");
const basePrefix = baseTag[0] && baseTag[0].href || "/";
const ROOT_URL = basePrefix.replace(window.location.origin, "");
const router = writable({
  path: "/",
  query: {},
  params: {},
  initial: true
});
const CTX_ROUTER = {};
const CTX_ROUTE = {};
let HASHCHANGE = window.location.origin === "null";
function hashchangeEnable(value) {
  if (typeof value === "boolean") {
    HASHCHANGE = !!value;
  }
  return HASHCHANGE;
}
function fixedLocation(path, callback, doFinally) {
  const baseUri = HASHCHANGE ? window.location.hash.replace("#", "") : window.location.pathname;
  if (path.charAt() !== "/") {
    path = baseUri + path;
  }
  const currentURL = baseUri + window.location.hash + window.location.search;
  if (currentURL !== path) {
    callback(path);
  }
  if (typeof doFinally === "function") {
    doFinally();
  }
}
function cleanPath(uri, fix) {
  return uri !== "/" || fix ? uri.replace(/\/$/, "") : uri;
}
function navigateTo(path, options) {
  const {
    reload,
    replace,
    params,
    queryParams
  } = options || {};
  if (!path || typeof path !== "string" || path[0] !== "/" && path[0] !== "#") {
    throw new Error(`Expecting '/${path}' or '#${path}', given '${path}'`);
  }
  if (params) {
    path = path.replace(/:([a-zA-Z][a-zA-Z0-9_-]*)/g, (_, key) => params[key]);
  }
  if (queryParams) {
    const qs = stringify(queryParams);
    if (qs) {
      path += `?${qs}`;
    }
  }
  if (HASHCHANGE) {
    let fixedURL = path.replace(/^#|#$/g, "");
    if (ROOT_URL !== "/") {
      fixedURL = fixedURL.replace(cleanPath(ROOT_URL), "");
    }
    window.location.hash = fixedURL !== "/" ? fixedURL : "";
    return;
  }
  if (reload || !window.history.pushState || !window.dispatchEvent) {
    window.location.href = path;
    return;
  }
  fixedLocation(path, (nextURL) => {
    window.history[replace ? "replaceState" : "pushState"](null, "", nextURL);
    window.dispatchEvent(new Event("popstate"));
  });
}
function getProps(given, required) {
  const { props: sub, ...others } = given;
  required.forEach((k) => {
    delete others[k];
  });
  return {
    ...sub,
    ...others
  };
}
function isActive(uri, path, exact) {
  if (!cache[[uri, path, exact]]) {
    if (exact !== true && path.indexOf(uri) === 0) {
      cache[[uri, path, exact]] = /^[#/?]?$/.test(path.substr(uri.length, 1));
    } else if (uri.includes("*") || uri.includes(":")) {
      cache[[uri, path, exact]] = Router.matches(uri, path);
    } else {
      cache[[uri, path, exact]] = cleanPath(path) === uri;
    }
  }
  return cache[[uri, path, exact]];
}
function isPromise(object) {
  return object && typeof object.then === "function";
}
function isSvelteComponent(object) {
  return object && object.prototype;
}
export {
  CTX_ROUTE,
  CTX_ROUTER,
  HASHCHANGE,
  ROOT_URL,
  cleanPath,
  fixedLocation,
  getProps,
  hashchangeEnable,
  isActive,
  isPromise,
  isSvelteComponent,
  navigateTo,
  router
};
